@@ -32,22 +32,37 @@ def setup( ) :
 
 #####################################################################
 
+def fsize( ) :
+    fp = filepath_get( )
+
+    fs = os.path.getsize( fp )
+
+    return( fs )
+    #os.path.getsize( fp )
+
+def fsize_limited( ) :
+    fs=fsize()
+    if( fs > 20971520 ) :
+        s_log.write(fs)
+        return( True )
+    return(False)
 
 def db_stash( ) :
+
     curr_fp = filepath_get( )
     new_fp = curr_fp + "-STASH"
     os.rename( curr_fp , new_fp )
-    data={
+    
+    data = {
         "curr_fp":curr_fp,
         "new_fp":new_fp
     }
-    return(s_util.json_encode(data))
+
+    return( s_util.json_encode( data ) )
 
 def db_unstash( data_json ) :
-    data=s_util.json_loads(data_json)
-    os.rename( data["new_fp"] , data["curr_fp"])
-
-
+    data = s_util.json_loads( data_json )
+    os.rename( data[ "new_fp" ] , data[ "curr_fp" ] )
 
 #####################################################################
 
@@ -60,20 +75,6 @@ def filedir_get( ) :
 
 #####################################################################
 
-
-# FIXME TODO when main error occurs
-#sapirunner_rbfan-6aARaqVjA_1  | ValueError: Port could not be cast to integer value as '172.17.0.1:10443'
-#sapirunner_rbfan-6aARaqVjA_1  | Error in atexit._run_exitfuncs:
-#sapirunner_rbfan-6aARaqVjA_1  | Traceback (most recent call last):
-#sapirunner_rbfan-6aARaqVjA_1  |   File "/usr/local/lib/python3.8/dist-packages/sapiadapter/s_cleanup.py", line 27, in cleanup
-#sapirunner_rbfan-6aARaqVjA_1  |     s_db.filepath_delete( )
-#sapirunner_rbfan-6aARaqVjA_1  |   File "/usr/local/lib/python3.8/dist-packages/sapiadapter/s_db.py", line 57, in filepath_delete
-#sapirunner_rbfan-6aARaqVjA_1  |     fp = filepath_get( )
-#sapirunner_rbfan-6aARaqVjA_1  |   File "/usr/local/lib/python3.8/dist-packages/sapiadapter/s_db.py", line 49, in filepath_get
-#sapirunner_rbfan-6aARaqVjA_1  |     dbcon = sqlite3.connect( fp )
-#sapirunner_rbfan-6aARaqVjA_1  | sqlite3.OperationalError: unable to open database file
-
-
 def filepath_get( ) :
     tid = str( s_util.thread_get_id( ) )
     fn = s_config.get_key( "/config/session" ) + "-" + tid + ".db"
@@ -83,7 +84,7 @@ def filepath_get( ) :
         dbcon = sqlite3.connect( fp )
         dbcon.commit( )
         dbcon.close( )
-        init_schema()
+        init_schema( )
         init_data( )
     return( fp )
 
@@ -93,8 +94,8 @@ def filepath_delete( ) :
         os.remove( fp )
 
 def filepath_delete_all( ) :
-    for f in os.scandir(filedir_get()):
-        os.remove(f.path)
+    for f in os.scandir( filedir_get( ) ) :
+        os.remove( f.path )
 
 #####################################################################
 
@@ -138,8 +139,11 @@ def sql_execute_fetch( sql , fetch_all=False) :
 
 #####################################################################
 
+
 def flush( ) :
 
+    # FIXME TODO Maybe delete file?
+
     table_truncate( "files" ) 
     table_truncate( "env" ) 
     table_truncate( "stdio" ) 
@@ -344,8 +348,7 @@ def stacks_read( sk = 0 ) :
     return( rows )
 
 def stacks_readall( ) :
-    # FIXME TODO ORDER by sid!
-    rows = table_readall( "stacks" )
+    rows = table_readall( "stacks" , extra = "order by sid asc" )
     return( rows )
 
 def stacks_update( sid , sk , sv_data ) :
@@ -359,8 +362,12 @@ def stacks_delete( sk = 0 ) :
     sql_execute( sql )
     return( True )
 
-def stacks_deleteall( ) :
-    table_truncate( "stacks" )
+# FIXME TODO finish when specific sk is given
+def stacks_deleteall( sk = False ) :
+    if( sk == False ) :
+        table_truncate( "stacks" )
+    else:
+        pass
     return(True)
 
 def stacks_pop( sk = 0 ) :
@@ -650,55 +657,84 @@ def files_dict2json( fp , fdict ) :
     fs = len( fc )
     fh = hashlib.md5( fc.encode( "utf-8" ) ).hexdigest( )
 
-    # FIXME TODO add fm
+    fm = s_util.time_now( )
 
-    sql = "INSERT INTO files ( fp , fc , fs , fh ) VALUES (?, ?, ?, ?)"
-    data_tuple = ( fp , fc , fs , fh )
+    sql = "INSERT INTO files ( fp , fc , fs , fh ,fm) VALUES (?, ?, ?, ?,?)"
+    data_tuple = ( fp , fc , fs , fh ,fm)
     sql_execute( sql , data_tuple )
 
     return( True )
 
 def files_getcontent( fp ) :
     f = files_read( fp )
-    if not f:
+    if not f :
         s_log.write( "False files_getcontent" )
         return( False )
     return( f[ "fc" ] )
 
-# with open( dir_cache + "local_out.png" , "wb" ) as f :
-def files_write( fp ) :
-    # FIXME TODO test this... with open( dir_cache + "local_out.png" , "wb" ) as f :
-    with open( fp , "w" , 1 ) as f :
-        f.write( files_getcontent( fp ) )
-
 def files_readallmeta( ) :
 
     sql = "SELECT fp,fs,fh,fm FROM files" 
     rows = sql_execute_fetch( sql , fetch_all = True )
     return( rows )
 
-def files_restore( ) :
+def files_write( fp_source , fp_target = False ) :
+
+    if( fp_target == False ) :
+        fp_target = fp_source
+
+    fd = files_read( fp_source )
+
+    fp_target_tmp = fp_target + s_util.uhash( )
+
+    with open( fp_target_tmp , "wb" ) as f :
+        f.write( fd[ "fc" ] )
+
+    s_util.file_replace( fp_target_tmp , fp_target )
+
+    s_util.file_settimes( fp_target , fd[ "fm" ] )
+
+
+# with open( dir_cache + "local_out.png" , "wb" ) as f :
+#def files_write( fp ) :
+#    # FIXME TODO test this... with open( dir_cache + "local_out.png" , "wb" ) as f :
+#    with open( fp , "w" , 1 ) as f :
+#        f.write( files_getcontent( fp ) )
+
+def files_restore( rpath = False ) :
     db_files = files_readallmeta( )
     s_log.write(db_files)
     for db_file in db_files :
 
-        fp = db_file[ "fp" ]
+        fp_source = db_file[ "fp" ]
 
-        dp = s_util.path_dirname( fp )
+        if(rpath!=False):
+            fp_target=rpath.rstrip("/")+ "/" + fp_source
+        else:
+            fp_target=fp_source
+
+        s_log.write(fp_source+" -> "+fp_target)
+
+        dp = s_util.path_dirname( fp_target )
         #print(dp)
         dp_path = Path( dp )
         dp_path.mkdir( parents = True , exist_ok = True )
 
-        with open( fp , "wb" ) as f :
-            f.write( files_getcontent( fp ) )
+
+        files_write( fp_source,fp_target)
+
+#        with open( fp_target , "wb" ) as f :
+#            f.write( files_getcontent( fp_source ) )
+
+#        s_util.file_settimes(fp_target,db_file["fm"])
 
 def files_createall( pat ) :
     # Let's be clever and try to trim the pattern matching directory...
 
     # FIXME TODO be careful... we are only checking for wildcard "**"
-    pat_trim=pat.rstrip("**")
-    if(pat_trim==pat):
-        pat_trim=""
+    pat_trim = pat.rstrip( "**" )
+    if( pat_trim == pat ) :
+        pat_trim = ""
 
     files = s_util.glob_files( pat , True )
     for f in files :
