@@ -26,6 +26,7 @@ import gzip
 import inspect
 import glob
 import traceback
+import random
 
 import threading
 
@@ -102,11 +103,24 @@ def json_dumps( inputs ) :
 #    print(pretty_object)
 
 def json_prettyprint( data ) :
-    pretty_object = json.dumps(data, indent=4)
-    print(pretty_object)
+    pretty_object = json.dumps( data , indent = 4 )
+    print( pretty_object )
 
 #####################################################################
 
+def num_randomlist( s = 10 ) :
+    random_list = [random.random() for i in range(s)]
+    return(random_list)
+
+# Get arg from position and use default
+def argv_get( p , d ) :
+    al=len( sys.argv )-1
+    print(al)
+    if(p>al): return(d)
+
+
+    return(sys.argv[p])
+
 # FIXME TODO aaarrggghhh!!!
 def time_getisoformat( ) :
 
@@ -177,12 +191,24 @@ def pathnotallowed( p ) :
 
     return( True )
 
+def save_hashkeycontent( fk , fc ) :
+    fp = "/tmp/" + hash( fk ) + ".tmp"
+    with open( fp , "w" ) as file:
+        file.write( fc )
+
+def load_hashkeycontent( fk ) :
+    fp = "/tmp/" + hash( fk ) + ".tmp"
+    if(file_exists(fp)):
+        return(file_read(fp))
+    return(False)
+
+
 
 def uhash( prefix = "" ) :
     return( prefix + hashlib.sha256( str( uuid.uuid4( ) ).encode( "utf-8" )  ).hexdigest( ) )
 
 def hash( str , prefix = "" ) :
-    return( prefix + hashlib.sha256( str ).hexdigest( ) )
+    return( prefix + hashlib.sha256( str.encode( "utf-8" ) ).hexdigest( ) )
 
 def hash_file_md5( fp ) :
     return( hashlib.md5( open( fp , "rb" ).read( ) ).hexdigest( ) )
@@ -556,6 +582,8 @@ def file_getcontent( fn , scp ) :
     return( contents )
 
 
+def file_read(fp):
+    return(open( fp , "r" ).read( ))
 
 def file_exists( fp ) :
     return( os.path.isfile(fp) )
@@ -582,6 +610,13 @@ def sys_path_append_fp( fp ) :
 def file_stat( fp ) :
     return( os.stat( fp ) )
 
+def file_settimes( fp , t ) :
+    os.utime( fp , ( t , t ) )
+
+
+def file_replace( fp_source , fp_target ):
+    os.replace(fp_source,fp_target)
+
 
 ################
 
